import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import automata.Automata;

public class AutomataApplet extends JApplet implements ActionListener, ChangeListener, StepListener, Runnable{
	private AutomataPanel automataPanel = null;

	private JPanel bottomPanel = new JPanel();
	private JButton resetButton = new JButton("Reset");
	private JLabel stepLabel = new JLabel();

	private SpinnerNumberModel spinModel = null;
	private JSpinner speedSpin = new JSpinner();

	private String currentAutomata = "";
	private JComboBox automataCombo = new JComboBox();

	private Thread runner = null;
	private boolean isRunning = true;

	public AutomataApplet(){
	}

	public void start(){
		if(runner == null){
			runner = new Thread(this);
			runner.start();
		}
	}

	public void stop(){
		if(runner != null){
			isRunning = false;
			runner = null;
		}
	}

	public void run(){
		requestFocus();
		isRunning = true;
		while(isRunning){
			try{
				Thread.sleep(1000);
			}
			catch(Exception e){
			}
		}
	}

	public void init(){
		automataPanel = new AutomataPanel(getWidth(), getHeight());

		getContentPane().add(automataPanel, BorderLayout.CENTER);
		getContentPane().add(bottomPanel, BorderLayout.SOUTH);

		spinModel = new SpinnerNumberModel(automataPanel.getSpeed(), 0, 500, 1);
		speedSpin = new JSpinner(spinModel);
		speedSpin.addChangeListener(this);

		resetButton.setMnemonic(KeyEvent.VK_R);
		resetButton.addActionListener(this);

		stepLabel.setPreferredSize(new Dimension(100, 20));
		stepLabel.setHorizontalAlignment(JLabel.RIGHT);

		listAutomata();
		updateAutomata();

		automataCombo.addActionListener(this);

		bottomPanel.add(speedSpin);
		bottomPanel.add(resetButton);
		bottomPanel.add(stepLabel);
		bottomPanel.add(automataCombo);

		automataPanel.addStepListener(this);
	}

	private void listAutomata(){
		automataCombo.removeAllItems();

		automataCombo.addItem("Ant");
		automataCombo.addItem("Life");
		automataCombo.addItem("Simple");
		automataCombo.addItem("GA");
		automataCombo.addItem("Stars");

		automataCombo.setSelectedIndex(0);
	}

	private void updateAutomata(){
		String automata = (String)automataCombo.getSelectedItem();
		if(automata != null && !automata.equals(currentAutomata)){
			currentAutomata = automata;

			try{
				String className = "automata." + automata + "." + automata;
//				Automata a = (Automata)(Class.forName(className)).newInstance();
				Automata a = (Automata)(getClass().getClassLoader().loadClass(className)).newInstance();
				if(a != null)
					automataPanel.setAutomata(a);
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	public void actionPerformed(ActionEvent event){
		Object src = event.getSource();
		if(src.equals(resetButton))
			automataPanel.resetAutomata();
		else if(src.equals(automataCombo))
			updateAutomata();
	}

	public void stateChanged(ChangeEvent event){
		if(event.getSource().equals(speedSpin))
			automataPanel.setSpeed(spinModel.getNumber().intValue());
	}

	public void stepPerformed(int step){
		stepLabel.setText(Integer.toString(step));
	}
}

// vim:noet:ts=3
