import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import automata.Automata;

public class AutomataPanel extends JPanel implements ActionListener{
	private int width  = 0;
	private int height = 0;

	private int updateDelay = 10; //milliseconds

	private Image buffer = null;
	private Graphics bg  = null;
	private Timer timer  = null;

	private Automata automata = null;

	private int aWidth, aHeight;
	private double deltax, deltay;

	private int steps = 0;

	private Vector stepListeners = new Vector();

	public AutomataPanel(int width, int height){
		this.width = width;
		this.height = height;
	}

	public void actionPerformed(ActionEvent event){
		if(automata != null){
			steps++;
			broadcastStepEvent();
			automata.step();
			updateBuffer();
			repaint(getVisibleRect());
		}
	}

	private void createBuffer(){
		deltax = (double)width / aWidth;
		deltay = (double)height / aHeight;

		buffer = createImage(width, height);

		if(buffer != null){
			bg = buffer.getGraphics();
			updateBuffer();
			timer = new Timer(updateDelay, this);
			timer.start();
		}
	}

	private void updateBuffer(){
//		bg.clearRect(0, 0, width, height);
		bg.setColor(Color.BLACK);
		bg.fillRect(0, 0, width, height);

		for(int i = 0; i < aWidth; i++){
			for(int j = 0; j < aHeight; j++){
				bg.setColor(automata.getColor(i, j));
				int x = (int)(i * deltax);
				int y = (int)(j * deltay);
				bg.fillRect(x, y, (int)deltax, (int)deltay);
			}
		}
	}

	public void paint(Graphics g){
		int w = getWidth();
		int h = getHeight();
		if(w != width || h != height || buffer == null){
			width = w;
			height = h;
			createBuffer();
		}

		if(buffer != null)
			g.drawImage(buffer, 0, 0, this);
	}

	public void resetAutomata(){
		steps = 0;
		broadcastStepEvent();

		if(automata != null)
			automata.reset();
	}

	public int getSpeed(){
		return updateDelay;
	}

	public void setSpeed(int delay){
		updateDelay = delay;
		timer.stop();
		timer = new Timer(updateDelay, this);
		timer.start();
	}

	public void setAutomata(Automata a){
		automata = a;
		aWidth = automata.getWidth();
		aHeight = automata.getHeight();
		resetAutomata();
	}

	public void addStepListener(StepListener listener){
		stepListeners.addElement(listener);
	}

	public void removeStepListener(StepListener listener){
		stepListeners.removeElement(listener);
	}

	private void broadcastStepEvent(){
		int numListeners = stepListeners.size();
		for(int i = 0; i < numListeners; i++)
			((StepListener)stepListeners.elementAt(i)).stepPerformed(steps);
	}
}

// vim:noet:ts=3
