package automata;

import java.util.*;
import java.awt.Color;

public abstract class Automata{
	protected int width = 100;
	protected int height = 100;

	protected int board[][] = new int[width][height];

	protected String className = "base";

	Vector palette = new Vector();

	public Automata(){
		setupPalette();
		reset();
	}

	public String getClassName(){
		return className;
	}

	void setupPalette(){
		int inc = 50;

		palette = new Vector();

		palette.addElement(new Color(0, 0, 0));

		for(int i = inc; i <= 255; i += inc)
			palette.addElement(new Color(0, 0, i));
		for(int i = 0; i <= 255; i += inc)
			palette.addElement(new Color(0, i, 255));
		for(int i = 255; i >= 0; i -= inc)
			palette.addElement(new Color(0, 255, i));
		for(int i = 0; i <= 255; i += inc)
			palette.addElement(new Color(i, 255, 0));
		for(int i = 255; i >= 0; i -= inc)
			palette.addElement(new Color(255, i, 0));
		for(int i = 0; i <= 255; i += inc)
			palette.addElement(new Color(255, 0, i));
		for(int i = 0; i <= 255; i += inc)
			palette.addElement(new Color(255, i, 255));

		palette.addElement(new Color(255, 255, 255));
	}

	protected void nullBoard(){
		for(int i = 0; i < width; i++)
			Arrays.fill(board[i], 0);
	}

	protected void uniformBoard(){
		int count = 0;
		for(int i = 0; i < width; i++)
			for(int j = 0; j < height; j++)
				board[i][j] = count++;
	}

	protected void randomBoard(){
		randomBoard(1);
	}

	protected void randomBoard(int init){
		Random random = new Random(System.currentTimeMillis());
		for(int i = 0; i < width; i++){
			for(int j = 0; j < height; j++){
				if(random.nextDouble() > 0.9)
					board[i][j] = init;
				else
					board[i][j] = 0;
			}
		}
	}

	public abstract void reset();
	public abstract void step();

	public int getWidth(){
		return width;
	}

	public int getHeight(){
		return height;
	}

	public int getStatus(int x, int y){
		return board[x][y];
	}

	public Color getColor(int x, int y){
		int status = board[x][y];

		if(status < 0)
			status = 0;
		else if(status >= palette.size())
			status = palette.size() - 1;

		return (Color)palette.elementAt(status);
	}
}

// vim:noet:ts=3
