package automata.GA;

import automata.Automata;

import java.util.Random;
import java.util.Arrays;

public class GA extends Automata{
	private static final int MAX_POP = 10;

	private int currentStep = 0;
	private NeuralNet population[] = new NeuralNet[MAX_POP];

	private Random random = new Random(System.currentTimeMillis());

	public void reset(){
		currentStep = 0;
		randomBoard(NeuralNet.MARK_TREE);

		random = new Random(System.currentTimeMillis());
		population = new NeuralNet[MAX_POP];
		for(int i = 0; i < MAX_POP; i++){
			population[i] = new NeuralNet();
			population[i].setPosition(random.nextInt(width), random.nextInt(height));
		}
	}

	private void evolve(){
		Arrays.sort(population, new NetCompare());

		double maxFitness = population[0].getFitness();

		System.out.println("Best fitness = " + maxFitness);

		NeuralNet newPop[] = new NeuralNet[MAX_POP];

		for(int i = 0; i < MAX_POP; i++){
			if((population[i].getFitness() / maxFitness) > 0.8)
				newPop[i] = population[i];
			else
				newPop[i] = population[random.nextInt(MAX_POP)];
		}

		for(int i = 0; i < MAX_POP; i += 2){
			population[i] = newPop[i];
			population[i].crossover(newPop[i + 1]);
			population[i].mutate();
			population[i].setPosition(random.nextInt(width), random.nextInt(height));
			population[i].setFitness(0);
		}

		randomBoard(NeuralNet.MARK_TREE);
	}

	public void step(){
		currentStep++;

		if((currentStep % 100) == 0)
			evolve();

		for(int i = 0; i < MAX_POP; i++)
			population[i].evaluate(board, width, height);
	}
}

// vim:noet:ts=3
