package automata.GA;

import java.util.Random;

public class NeuralNet{
	private int x = 0;
	private int y = 0;

	private double fitness = 0;

	private int numInputs = 8;
	private int numHidden = 13;
	private int numOutput = 9;

	private double inWeights[][] = null;
	private double hiddenWeights[][] = null;

	public static final int MARK_BOID = 10;
	public static final int MARK_TREE = 15;

	private static Random random = new Random(System.currentTimeMillis());

	public NeuralNet(){
		inWeights = new double[numInputs][numHidden];
		hiddenWeights = new double[numHidden][numOutput];

		randomNet();
	}

	private void randomNet(){
		for(int i = 0; i < numInputs; i++)
			for(int j = 0; j < numHidden; j++)
				inWeights[i][j] = random.nextGaussian();

		for(int i = 0; i < numHidden; i++)
			for(int j = 0; j < numOutput; j++)
				hiddenWeights[i][j] = random.nextGaussian();
	}

	public void mutate(){
		for(int i = 0; i < numInputs; i++)
			for(int j = 0; j < numHidden; j++)
				inWeights[i][j] += 0.01 * random.nextGaussian();

		for(int i = 0; i < numHidden; i++)
			for(int j = 0; j < numOutput; j++)
				hiddenWeights[i][j] += 0.01 * random.nextGaussian();
	}

	public void crossover(NeuralNet net){
		int range = numInputs * numHidden + numHidden * numOutput;
		int start = random.nextInt(range);
		int end = random.nextInt(range - start) + start;

		int count = 0;

		for(int i = 0; i < numInputs; i++){
			for(int j = 0; j < numHidden; j++, count++){
				if(count >= start && count <= end)
					inWeights[i][j] = net.inWeights[i][j];
			}
		}

		for(int i = 0; i < numHidden; i++){
			for(int j = 0; j < numOutput; j++, count++){
				if(count >= start && count <= end)
					hiddenWeights[i][j] = net.hiddenWeights[i][j];
			}
		}
	}

	private double sigmoid(double x){
		return (1.0 / (1.0 + Math.exp(-x)));
	}

	private void activate(double[] inputs, double[] outputs){
		double hidden[] = new double[numHidden];

		for(int j = 0; j < numHidden; j++){
			hidden[j] = 0;
			for(int i = 0; i < numInputs; i++)
				hidden[j] += inputs[i] * inWeights[i][j];
			hidden[j] = sigmoid(hidden[j]);
		}

		for(int j = 0; j < numOutput; j++){
			outputs[j] = 0;
			for(int i = 0; i < numHidden; i++)
				outputs[j] += hidden[i] * hiddenWeights[i][j];
			outputs[j] = sigmoid(outputs[j]);
		}
	}

	public void evaluate(int[][] board, int width, int height){
		board[x][y] = 0;

		double inputs[] = new double[numInputs];
		double outputs[] = new double[numOutput];

		int x0 = clip(x - 1, width);
		int x1 = clip(x + 1, width);
		int y0 = clip(y - 1, height);
		int y1 = clip(y + 1, height);

		inputs[0] = checkState(board[x0][y0]);
		inputs[1] = checkState(board[x ][y0]);
		inputs[2] = checkState(board[x1][y0]);
		inputs[3] = checkState(board[x0][y ]);
		inputs[4] = checkState(board[x1][y ]);
		inputs[5] = checkState(board[x0][y1]);
		inputs[6] = checkState(board[x ][y1]);
		inputs[7] = checkState(board[x1][y1]);

		activate(inputs, outputs);

		int index = -1;
		double maxOutputs = -1000;
		for(int i = 0; i < numOutput; i++){
			if(outputs[i] > maxOutputs){
				maxOutputs = outputs[i];
				index = i;
			}
		}

		if(index < 0)
			System.err.println("No output detected...");
		else{
			switch(index){
				case 0:
					break;
				case 1:
					x--;
					break;
				case 2:
					x++;
					break;
				case 3:
					y--;
					break;
				case 4:
					y++;
					break;
				case 5:
					x--; y--;
					break;
				case 6:
					x--; y++;
					break;
				case 7:
					x++; y--;
					break;
				case 8:
					x++; y++;
					break;
				default:
					System.err.println("Invalid index...");
					break;
			}

			x = clip(x, width);
			y = clip(y, height);

		}

		if(board[x][y] == MARK_BOID)
			fitness -= 0.1;
		else if(board[x][y] == MARK_TREE)
			fitness += 1.0;

		board[x][y] = MARK_BOID;
	}
	
	private int clip(int x, int size){
		if(x >= size)
			x -= size;
		if(x < 0)
			x += size;

		return x;
	}

	private double checkState(int pos){
		switch(pos){
			case MARK_BOID:
				return 0.0;
			case MARK_TREE:
				return 1.0;
			default:
				return 0.5;
		}
	}

	public int getX(){
		return x;
	}

	public int getY(){
		return y;
	}

	public void setPosition(int newx, int newy){
		x = newx;
		y = newy;
	}

	public double getFitness(){
		return fitness;
	}

	public void setFitness(double newFitness){
		fitness = newFitness;
	}
}

// vim:noet:ts=3
